#ifndef ATRMEMORYIMAGE_H
#define ATRMEMORYIMAGE_H

/*
   AtrMemoryImage.h - access ATR images in RAM.

   Copyright (C) 2002 Matthias Reichl <hias@horus.com>

   This program is free software; you can redistribute it and/or modify
   it under the terms of the GNU General Public License as published by
   the Free Software Foundation; either version 2 of the License, or
   (at your option) any later version.

   This program is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
   GNU General Public License for more details.

   You should have received a copy of the GNU General Public License
   along with this program; if not, write to the Free Software
   Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
*/

#include "AtrImage.h"

class AtrMemoryImage : public AtrImage {
public:

	AtrMemoryImage();

	virtual ~AtrMemoryImage();

	virtual bool CreateImage(EDiskFormat format);
	virtual bool CreateImage(ESectorLength density, size_t sectors);
	virtual bool CreateImage(ESectorLength density, size_t sectorsPerTrack, size_t tracks, size_t sides);

	void FreeImageData();

	bool ReadImageFromFile(const char* filename);
	bool WriteBackImageToFile() const;
	bool WriteImageToFile(const char* filename) const;

	bool ReadSector(size_t sector,
		       unsigned char* buffer,
		       size_t buffer_length) const;

	bool WriteSector(size_t sector,
		       const unsigned char* buffer,
		       size_t buffer_length);

	bool Changed() const;

	virtual bool IsAtrMemoryImage() const;
	virtual void SetWriteProtect(bool on);

	friend class DCMCodec;

protected:

private:

	enum EImageType {
		eAtrImageType = 0,
		eAtrGzImageType = 1,
		eXfdImageType = 2,
		eXfdGzImageType = 3,
		eDcmImageType = 4,
		eDcmGzImageType = 5,
		eDiImageType = 6,
		eDiGzImageType = 7,
		eUnknownImageType=99
	};

	EImageType DetermineImageTypeFromFilename(const char* filename) const;

	bool ReadImageFromAtrFile(const char* filename);
	bool WriteImageToAtrFile(const char* filename, const bool useGz) const;

	bool ReadImageFromXfdFile(const char* filename);
	bool WriteImageToXfdFile(const char* filename, const bool useGz) const;

	bool ReadImageFromDcmFile(const char* filename);
	bool WriteImageToDcmFile(const char* filename, const bool useGz) const;

	bool ReadImageFromDiFile(const char* filename);
	bool WriteImageToDiFile(const char* filename, const bool useGz) const;

	unsigned char CalculateDiSectorChecksum(unsigned char* buf, size_t len) const;

	typedef AtrImage super;

	unsigned char *fData;

	mutable bool fChanged;

};

inline bool AtrMemoryImage::Changed() const
{
	return fChanged;
}
#endif
