#include <stdlib.h>
#include <limits.h>

#include "DiskSlot.h"
#include "AtrMemoryImage.h"

#include "AtariDebug.h"


DiskSlot::DiskSlot()
	: fInUse(false),
	  fFilename(0),
	  fHandler(0)
{
}

DiskSlot::~DiskSlot()
{
	if (fInUse) {
		UnloadImage();
	}
}

int DiskSlot::LoadAtrMemoryImage(const char* filename)
{
	char absPath[PATH_MAX];

	if (realpath(filename,absPath) == 0) {
		DPRINTF("get realpath failed\n");
		return 1;
	}

	fFilename = new char[strlen(absPath)+1];
	strcpy(fFilename, absPath);

	AtrMemoryImage *img = new AtrMemoryImage;
	if (!img->ReadImageFromFile(fFilename)) {
		DPRINTF("cannot open \"%s\"\n", fFilename);
		delete[] fFilename;
		delete img;
		fFilename = 0;
		return 1;
	}

	AtrSIOHandler* handler = new AtrSIOHandler(img, true);
	handler->EnableHighSpeed(true);

	fHandler = handler;

	fInUse = true;
	return 0;
}

int DiskSlot::UnloadImage()
{
	if (fInUse) {
		if (fFilename) {
			delete[] fFilename;
			fFilename = 0;
		}
		if (fHandler) {
			delete fHandler;
			fHandler = 0;
		}
		fInUse = false;
	}
	return 0;
}

int DiskSlot::WriteBackImageIfChanged()
{
	if (fInUse) {
		if (fHandler->IsAtrSIOHandler()) {
			AtrSIOHandler* handler = static_cast<AtrSIOHandler*>(fHandler);
			if (handler->GetAtrImage()->IsAtrMemoryImage()) {
				AtrMemoryImage* img = static_cast<AtrMemoryImage*>(handler->GetAtrImage());
				if (img->Changed()) {
					if (!img->WriteImageToFile(fFilename)) {
						DPRINTF("writing \"%s\" failed\n", fFilename);
						return 1;
					} else {
						DPRINTF("wrote \"%s\"\n",fFilename);
					}
				} else {
					DPRINTF("\"%s\" unchanged, not writing back\n",fFilename);
				}
			}
		}
	}
	return 0;
}
