/*
   DiskSlot - data structure holding information about an activated
   virtual disk drive

   Copyright (C) 2002 Matthias Reichl <hias@horus.com>

   This program is free software; you can redistribute it and/or modify
   it under the terms of the GNU General Public License as published by
   the Free Software Foundation; either version 2 of the License, or
   (at your option) any later version.

   This program is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
   GNU General Public License for more details.

   You should have received a copy of the GNU General Public License
   along with this program; if not, write to the Free Software
   Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
*/

#ifndef DISKSLOT_H
#define DISKSLOT_H

#include "AtrImage.h"
#include "AtrSIOHandler.h"

class DiskSlot {
public:
	DiskSlot();
	~DiskSlot();

	int LoadAtrMemoryImage(const char* filename);
	int WriteBackImageIfChanged();

	int UnloadImage();

	bool IsInUse() const;

	AtrSIOHandler* GetSIOHandler();
	const char* GetFilename() const;

private:

	bool fInUse;
	char* fFilename;
	AtrSIOHandler* fHandler;
};

inline bool DiskSlot::IsInUse() const 
{
	return fInUse;
}       

inline AtrSIOHandler* DiskSlot::GetSIOHandler()
{
	return fHandler;
}

inline const char* DiskSlot::GetFilename() const
{
	return fFilename;
}
#endif
