#ifndef FILEIO_H
#define FILEIO_H

#include <stdio.h>
#include <sys/types.h>

#ifdef USE_ZLIB
#include <zlib.h>
#endif

class FileIO
{
public:
	FileIO();
	virtual ~FileIO();

	virtual bool OpenRead(const char* filename) = 0;
	virtual bool OpenWrite(const char* filename) = 0;

	virtual bool Close() = 0;

	virtual size_t ReadBlock(void* buf, size_t len) = 0;
	virtual size_t WriteBlock(const void* buf, size_t len) = 0;

	virtual size_t GetFileLength() = 0;
	virtual size_t Tell() = 0;

	bool ReadByte(unsigned char& byte);
	bool WriteByte(const unsigned char& byte);

	/* read/write lo, hi byte */
	bool ReadWord(unsigned int& word);
	bool WriteWord(const unsigned int& word);

	/* read/write hi, lo byte */
	bool ReadBigEndianWord(unsigned int& word);
	bool WriteBigEndianWord(const unsigned int& word);

	virtual bool Unlink(const char* filename);

	virtual bool IsOpen() const = 0;

private:
	bool fIsOpen;
};

class StdFileIO : public FileIO
{
public:
	StdFileIO();
	virtual ~StdFileIO();

	virtual bool OpenRead(const char* filename);
	virtual bool OpenWrite(const char* filename);

	virtual bool Close();

	virtual size_t ReadBlock(void* buf, size_t len);
	virtual size_t WriteBlock(const void* buf, size_t len);

	virtual size_t GetFileLength();
	virtual size_t Tell();

	virtual bool IsOpen() const;

private:
	typedef FileIO super;

	FILE* fFile;
};

#ifdef USE_ZLIB

class GZFileIO : public FileIO
{
public:
	GZFileIO();
	virtual ~GZFileIO();

	virtual bool OpenRead(const char* filename);
	virtual bool OpenWrite(const char* filename);

	virtual bool Close();

	virtual size_t ReadBlock(void* buf, size_t len);
	virtual size_t WriteBlock(const void* buf, size_t len);

	virtual size_t GetFileLength();
	virtual size_t Tell();

	virtual bool IsOpen() const;

private:
	typedef FileIO super;

	gzFile fFile;
};

#endif

#endif
